<?php

namespace Drupal\syncart\Controller;

use Drupal\Core\Url;
use Drupal\commerce_order\Entity\Order;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\Request;
use Drupal\commerce_cart\CartProviderInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Drupal\syncart\Service\SynCartServiceInterface;
use Drupal\syncart\Service\SynRenderServiceInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Cart Controller.
 */
class CartController extends ControllerBase {

  const CART_ROUTE = 'syncart.cart_controller_cart';

  /**
   * Current cart.
   *
   * @var \Drupal\syncart\Service\SynCartServiceInterface
   */
  protected $cart;

  /**
   * Cart Renderer.
   *
   * @var \Drupal\syncart\Service\SynRenderServiceInterface
   */
  protected $renderer;

  /**
   * Constructs a new CartController object.
   *
   * @param \Drupal\commerce_cart\CartProviderInterface $cart_provider
   *   Cart provider.
   * @param \Drupal\syncart\Service\SynCartServiceInterface $syncart
   *   Syncart cart.
   * @param \Drupal\syncart\Service\SynRenderServiceInterface $syncart_renderer
   *   Syncart Renderer.
   */
  public function __construct(
    CartProviderInterface $cart_provider,
    SynCartServiceInterface $syncart,
    SynRenderServiceInterface $syncart_renderer
  ) {
    /** @var \Drupal\syncart\Service\SynCartServiceInterface $cart */
    $this->cart = new $syncart($cart_provider);
    $this->renderer = $syncart_renderer;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('commerce_cart.cart_provider'),
      $container->get('syncart.cart'),
      $container->get('syncart.render')
    );
  }

  /**
   * Cart.
   */
  public function checkout() {
    if ($this->cart->isEmpty()) {
      return $this->redirect('syncart.cart_controller_cart');
    }
    $cart = $this->cart->load();
    $url = Url::fromRoute('commerce_checkout.form', [
      'commerce_order' => $cart->id(),
      'step' => 'order_information',
    ]);
    return new RedirectResponse($url->toString());
  }

  /**
   * Add product variation to cart.
   */
  public function addToCart(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      $this->cart->addItem($params['vid'], $params['qty']);
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Add variations to cart.
   */
  public function addToCartMultiple(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vids'])) {
      foreach ($params['vids'] as $vid) {
        $this->cart->addItem($vid, 1);
      }
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Задать количество товара в корзине.
   */
  public function setProductQuantity(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      if (!empty($params['quantity']) && round($params['quantity'], 2) > 0) {
        $vid = (int) $params['vid'];
        $quantity = round($params['quantity'], 2);
        $this->cart->setOrderItemQuantity($vid, $quantity);
      }
    }
    return new JsonResponse($this->renderer->render());
  }

  /**
   * Задать количество товара в корзине.
   */
  public function removeFromCart(Request $request) {
    $params = Json::decode($request->getContent());
    if (!empty($params['vid']) && (int) $params['vid'] > 0) {
      $this->cart->removeOrderItem($params['vid']);
    }
    return $this->redirect(self::CART_ROUTE);
  }

  /**
   * Получение текущей корзины.
   */
  public function loadCart($cid = FALSE) {
    return new JsonResponse($this->renderer->render($cid));
  }

  /**
   * {@inheritdoc}
   */
  public function repeatOrder(Order $commerce_order) {
    $this->cart->addOrderItem($commerce_order);
    return $this->redirect('syncart.cart_controller_cart');
  }

}
