<?php

namespace Drupal\syncart\Service;

use Drupal\commerce_cart\CartProviderInterface;
use Drupal\commerce_order\Entity\OrderItem;
use Drupal\commerce_store\Entity\Store;

/**
 * Custom Cart Service.
 */
class SynCartService implements SynCartServiceInterface {

  /**
   * Провайдер корзины.
   *
   * @var \Drupal\commerce_cart\CartProviderInterface
   */
  protected $cartProvider;

  /**
   * Объект текущей корзины.
   *
   * @var \Drupal\commerce_order\Entity\OrderInterface
   */
  protected $cart;

  /**
   * Constructs a new SynCartService object.
   */
  public function __construct(CartProviderInterface $cart_provider) {
    $this->cartProvider = $cart_provider;
    // Получение текущей корзины.
    $this->orderType = 'default';
    $this->cart = $this->cartProvider->getCart($this->orderType);
    // Проверка на наличие дублирующих корзин.
    $this->clearCarts();
    $this->vid = 0;
  }

  /**
   * Получить текущую корзину пользователя.
   */
  public function load() {
    return $this->cart;
  }

  /**
   * Добавить товар в корзину.
   */
  public function addItem($vid, $quantity = 1) {
    if (empty($this->cart)) {
      $this->createCart();
    }
    $this->vid = $vid;
    $variation = \Drupal::entityTypeManager()
      ->getStorage('commerce_product_variation')
      ->load($vid);
    $ccode = FALSE;
    if ($this->cart->total_price->number > 0) {
      $ccode = $this->cart->total_price->currency_code;
    }
    if (is_object($variation)) {
      if ($ccode && $ccode != $variation->price->currency_code) {
        \Drupal::messenger()->addError(t("Wrong Currency"));
        return $this->cart;
      }
      $this->setOrderItem($variation, $quantity);
    }
    return $this->cart;
  }

  /**
   * Удалить товар из корзины.
   */
  public function removeOrderItem($orderItemId) {
    $orderItem = OrderItem::load($orderItemId);
    if (is_object($orderItem)) {
      $this->cart->removeItem($orderItem);
      $orderItem->delete();
      return $this->cart->save();
    }
    return FALSE;
  }

  /**
   * Обновить количество товара в корзине.
   */
  public function setOrderItemQuantity($orderItemId, $quantity = 1) {
    if (empty($orderItemId) || !is_numeric($orderItemId)) {
      return FALSE;
    }
    if (is_object($orderItem = OrderItem::load($orderItemId))) {
      $orderItem->setQuantity($quantity);
      $this->cart->save();
      $this->cart->recalculateTotalPrice();
      return $this->cart->save();
    }
    return FALSE;
  }

  /**
   * Если в заказе есть вариация, увеличить кол-во, иначе добавить новую.
   */
  private function setOrderItem($variation, $quantity) {
    $hasItem = FALSE;
    if (!is_object($this->cart)) {
      return;
    }
    foreach ($this->cart->getItems() as $order_item) {
      if ($order_item->getPurchasedEntityId() == $variation->id()) {
        $newQuantity = round(($order_item->getQuantity() + $quantity), 2);
        $order_item->setQuantity($newQuantity);
        $order_item->save();
        $hasItem = TRUE;
        break;
      }
    }
    // Если вариация ранее не была добавлена, добавляем.
    if (!$hasItem) {
      /** @var \Drupal\commerce_order\OrderItemStorageInterface $order_item_storage */
      $order_item_storage = \Drupal::entityTypeManager()->getStorage('commerce_order_item');
      $order_item = $order_item_storage->createFromPurchasableEntity($variation, ['quantity' => $quantity]);
      if (is_object($order_item)) {
        $order_item->save();
        $this->cart->addItem($order_item);
      }
    }
    $this->cart->save();
  }

  /**
   * Объединение и очистка дублирующихся корзин.
   */
  private function clearCarts() {
    $carts = $this->cartProvider->getCarts();
    if (count($carts) > 1) {
      $goods = [];
      foreach ($carts as $key => $otherCart) {
        if ($otherCart->bundle() == $this->orderType && $otherCart->id() !== $this->cart->id()) {
          foreach ($otherCart->getItems() as $cartItem) {
            $otherCart->removeItem($cartItem);
            $pid = $cartItem->purchased_entity->target_id;
            $quantity = $cartItem->quantity->value;
            if (isset($goods[$pid])) {
              $goods[$pid] += $quantity;
            }
            else {
              $goods[$pid] = $quantity;
            }
          }
          $otherCart->delete();
        }
      }
      foreach ($goods as $pid => $quantity) {
        $this->addItem($pid, $quantity);
      }
    }
  }

  /**
   * Перевод корзины в статус заказа.
   */
  public function cartToOrder() {
    $this->cart->set('state', 'completed');
    $this->cart->save();
    return $this->cart;
  }

  /**
   * Создать новую корзину.
   */
  private function createCart() {
    $account = \Drupal::currentUser();
    $store = Store::load(1);
    $this->cart = $this->cartProvider->createCart('default', $store, $account);
  }

  /**
   * {@inheritdoc}
   */
  public function sendReceipt($order) {
    $our_mail = \Drupal::config('contact_mail.settings')->get('emails');
    $mail_handler = \Drupal::service('commerce.mail_handler');
    $order_total_summary = \Drupal::service('commerce_order.order_total_summary');
    $order_type_storage = \Drupal::entityTypeManager()->getStorage('commerce_order_type');
    /** @var \Drupal\commerce_order\Entity\OrderTypeInterface $order_type */
    $order_type = $order_type_storage->load($order->bundle());
    $profile = $order->getBillingProfile();
    $x = str_replace(["\r\n", "\r", "\n"], " ", $our_mail);
    $x = str_replace(",", " ", $x);
    $mails = array_diff(explode(' ', $x), ['', NULL, FALSE]);
    if (!is_object($profile)) {
      return;
    }
    $to = $profile->field_customer_email->value;
    if ((empty($mails) && (!$to || !$order_type->get('sendReceipt')))) {
      return;
    }

    $profile_view_builder = \Drupal::entityTypeManager()->getViewBuilder('profile');
    $subject = t('Order #@number confirmed', ['@number' => $order->getOrderNumber()]);
    $body = [
      '#theme' => 'commerce_order_receipt',
      '#order_entity' => $order,
      '#totals' => $order_total_summary->buildTotals($order),
    ];
    $body['#billing_information'] = $profile_view_builder->view($profile);
    $params = [
      'id' => 'order_receipt',
      'from' => $order->getStore()->getEmail(),
      'order' => $order,
    ];
    $customer = $order->getCustomer();
    if ($customer->isAuthenticated()) {
      $params['langcode'] = $customer->getPreferredLangcode();
    }
    foreach ($mails as $key => $value) {
      $mail_handler->sendMail($value, $subject, $body, $params);
    }
    if ($to && $order_type->get('sendReceipt')) {
      /** @var \Drupal\commerce_store\StoreStorage $storage */
      $storage = \Drupal::entityTypeManager()->getStorage('commerce_store');
      /** @var \Drupal\commerce_store\Entity\StoreInterface $store */
      $store = $storage->loadDefault();
      $params['reply-to'] = $store->getEmail();
      $mail_handler->sendMail($to, $subject, $body, $params);
    }
  }

  /**
   * Повторить заказ.
   */
  public function addOrderItem($commerce_order) {
    $cart_manager = \Drupal::service('commerce_cart.cart_manager');
    $this->order = $commerce_order;

    if (empty($this->cart)) {
      $this->createCart();
    }
    foreach ($this->order->getItems() as $order_item) {
      $variation = $order_item->getPurchasedEntity();
      $product = $variation ? $variation->getProduct() : NULL;
      if ($product && $product->isPublished()) {
        $order_item_new = $order_item->createDuplicate();
        $order_item_new->enforceIsNew();
        $order_item_new->id = NULL;
        $order_item_new->order_item_id = NULL;
        $order_item_new->save();
        $cart_manager->addOrderItem($this->cart, $order_item_new);
      }
      else {
        $message = "Some products weren't copied to the cart as they aren't currently available.";
      }
    }
  }

  /**
   * User Authorization.
   */
  public function updateUserRegister($registration) {
    $config = \Drupal::configFactory()->getEditable('user.settings');
    if (is_object($config)) {
      if ($registration) {
        $config->set('register', 'visitors');
      }
      else {
        $config->set('register', 'admin_only');
      }
      $config->save(TRUE);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function isEmpty() {
    $items = [];
    if (is_object($this->cart)) {
      $items = $this->cart->getItems();
    }
    return empty($items);
  }

}
